/*-
 * Copyright (C) @BABOLO  2002 Dec 17
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 17\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: cmd_copy.c,v 1.2 2002/12/23 02:04:38 babolo Exp $";
#endif /* not lint */
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgm.h"
#include "pgoblin.h"

enum fich {fn, fs, fp, fo, fx};
enum finde {finde_beg, finde_eol, finde_bsl, finde_dot, finde_end};
/* static const char *fichn[] = {"fn", "fs", "fp", "fo", "fx"};
 * static const char *finden[] = {"finde_beg", "finde_eol", "finde_bsl", "finde_dot", "finde_end"};
 */

int
copyout(struct connopt *options, FILE *in, char *query, FILE *out) {
    int k, f, u = 0;
    size_t i;
    PGresult *res;
    char *q, buf[BUFSIZE];

    if  (v_flag > 3) fprintf( stderr, "+copyout %s in=%s query=%s out=%s\n"
                            , options->query ? options->query : ""
                            , in ? "OK" : "NULL"
                            , query ? query : "NULL"
                            , out ? "OK" : "NULL"
                            );
    if  (shurecon(options))                                                                return(-1);

    if  (query) {                                                        /*  query   bang */
        if  (!(q = bangstrip(query, "copyout: No query after #")))                         return(-1);
    } else if (options->query) {             /*  options->query      */
        asprintf(&q, "COPY %s TO stdout", options->query);
        if  (q == NULL) {                   if (v_flag >= 0) warnx("copyout: No memory"); return(-1);}
        u = 1;
    } else if (in) {                                                  /*   in   bang */
        if  ((q = prefile(in, NULL, NULL)) == NULL)                                        return(-1);
        if  ((q = bangstrip(q, "copyout: No query after #")) == NULL)                      return(-1);
    } else {                                 if (v_flag >= 0) warnx("copyout: No query"); return(-1);}

    PQEXEC(res, q, PGRES_COPY_OUT, -1);
    if  (u) free(q);

    for (k = 0; (f = PQgetline(options->conn, buf, BUFSIZE)) != EOF && !feof(out) && !ferror(out);) {
        if  (v_flag > 3)                                  fprintf(stderr, "%d%d buf=%s\n", k, f, buf);
        if  (!k && strcmp("\\.", buf) == 0) break;
        if  (f == 1) {
            fwrite(buf, 1, BUFSIZE - 1, out);
        } else {
            i = strlen(buf);
            buf[i] = '\n';
            fwrite(buf, i + 1, 1, out);
        }
        k = f;
    }
    if  ((f = PQendcopy(options->conn)) != 0) {
        if (v_flag >= 0)        warnx("copyout: PQendcopy failed: %s", PQerrorMessage(options->conn));
        return(-1);
    }
    PQclear(res);
    if  (v_flag > 3)                                              fprintf(stderr, "-copyout %d\n", 0);
    return(0);
}

int
copyin(struct connopt *options, FILE *in, char *query, char *table) {
    int u = 0;                                              /*      */
    int e = 0;                                                  /*   PQputnbytes */
    size_t f = 0;
    PGresult *res;
    char *q, *d, bufq[BUFSIZE];
    enum finde k;
                                                  /*    "\n\\.\n"    */
    const enum fich fich[256] =
    {fn,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fn,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fp,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo

    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fs,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo

    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo

    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo ,fo,fo,fo,fo
    };
    const u_char automa[finde_end][fx] =
    /*      fn         fs         fp         fo */
    {{ finde_eol, finde_beg, finde_beg, finde_beg} /*finde_beg*/
    ,{ finde_beg, finde_bsl, finde_beg, finde_beg} /*finde_eol*/
    ,{ finde_beg, finde_beg, finde_dot, finde_beg} /*finde_bsl*/
    ,{ finde_end, finde_beg, finde_beg, finde_beg} /*finde_dot*/
    };

    if  (v_flag > 3) fprintf( stderr, "+copyin %s in=%s query=%s table=%s\n"
                            , options->query ? options->query : ""
                            , in ? "OK" : "NULL"
                            , query ? query : "NULL"
                            , table ? table : "NULL"
                            );
    if  (shurecon(options))                                                                return(-1);
    d = NULL;                                                                 /*   COPY */

    if  (query) {                                                        /*  query   bang */
        if  (!(q = bangstrip(query, "copyin: No query after #")))                          return(-1);
    } else if (options->query) {             /*  options->query      */
        asprintf(&q, "COPY %s FROM stdin", options->query);
        if  (q == NULL) {                  if (v_flag >= 0) warnx("copyin: No memory\n"); return(-1);}
        u = 1;
    } else if (in) {                                         /*   in   bang   */
        if  ((q = prefile(in, NULL, NULL)) == NULL)                                        return(-1);
        if  ((q = bangstrip(q, "copyin: No query after #")) == NULL)                       return(-1);
        d = index(q, '\n');         /*       .  -  */
        if  (d) *d++ = '\0';
    } else {                                  if (v_flag >= 0) warnx("copyin: No query"); return(-1);}

    PQEXEC(res, q, PGRES_COPY_IN, -1);
    if  (u) free(q);

    if  ((q = table) || (q = d)) {                                           /*     */
        if  (v_flag > 3)                                          fprintf(stderr, "(%d)=", strlen(q));
        for (k = finde_beg; *q && k != finde_end; k = automa[k][fich[(u_char)*q++]])
            if  (v_flag > 3)                                              fprintf(stderr, "%02X", *q);
        e = PQputnbytes(options->conn, table, q - table);
        if  (v_flag > 3)     fprintf(stderr, "\nPQputnbytes=%d %d %s(%d)\n", e, k, table, q - table);
    } else if  (in) {                           /*    in,     bang.        */
        int b = options->bang ? 2 : 0;          /*    __   . */
        char * buf = NULL;

        for (k = finde_beg; !feof(in) && !ferror(in) && e == 0;) {
            f = fread(bufq, 1, BUFSIZE, in);
            if  (v_flag > 3)                                          fprintf(stderr, "fread(%d,", f);
            buf = bufq;
            if  (b == 2) {if (*buf == '#') b = 1; else b = 0;};
            if  (b == 1) {                                       /*     bang -  */
                for (; f > 0;) {
                    f--;
                    if  (*buf++ == '\n') {
                        b = 0;
                        break;
            }   }   }
            if  (v_flag > 3)                                               fprintf(stderr, "%d)=", f);
            if  (b == 0) {
                for (q = buf; q - buf < f && k != finde_end; k = automa[k][fich[(u_char)*q++]])
                    if  (v_flag > 3)                                      fprintf(stderr, "%02X", *q);
                e = PQputnbytes(options->conn, buf, q - buf);
                if  (v_flag > 3)        fprintf(stderr, "\nPQputnbytes=%d %d (%d)\n", e, k, q - buf);
        }   }
    } else {                             if (v_flag >= 0) warnx("copyin: No input data"); return(-1);}

    if  (e == 0 && k != finde_end) {                      /*  ()    */
        const char *enstin = "\n\\.\n";
        if  (v_flag > 3)                    fprintf(stderr, "end=%s~%d\n", &enstin[k], finde_end - k);
        e = PQputnbytes(options->conn, &enstin[k], finde_end - k);
    }
    if  (e) {                                                  /*       */
        if (v_flag >= 0)       warnx("copyin: PQputnbytes failed: %s", PQerrorMessage(options->conn));
        return(-1);
    }
    if  ((f = PQendcopy(options->conn)) != 0) {                       /*  -  COPY */
        if (v_flag >= 0)         warnx("copyin: PQendcopy failed: %s", PQerrorMessage(options->conn));
        return(-1);
    }
    PQclear(res);
    if  (v_flag > 3)                                               fprintf(stderr, "-copyin %d\n", 0);
    return(0);
}
