/*-
 * Copyright (C) @BABOLO  2002 Dec 17
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 17\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: cmd_strselect.c,v 1.3 2002/12/23 02:04:38 babolo Exp $";
#endif /* not lint */
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgm.h"
#include "pgoblin.h"

PGresult *
strselect_q(struct connopt *options, FILE *in, char *query) {
    PGresult *res;
    char *q;

    if  (v_flag > 3) fprintf( stderr, "+strselect_q %s in=%s query=%s\n"
                            , options->query ? options->query : ""
                            , in ? "OK" : "NULL"
                            , query ? query : "NULL"
                            );
    if  (shurecon(options)) return(NULL);
    if  (query) q = query;
    else if (in) {
        if  ((q = prefile(in, NULL, NULL)) == NULL)                                      return(NULL);
    } else if (options->query) q = options->query;
    else {                               if (v_flag >= 0) warnx("strselect: No query"); return(NULL);}

    if  (*q == '#') {
        q = index(q, '\n');
        if  (!q) {               if (v_flag >= 0) warnx("strselect: No query after #"); return(NULL);}
        q++;
    }
    PQEXEC(res, q, PGRES_TUPLES_OK, NULL);
    if  (v_flag > 3)                       fprintf( stderr, "-strselect_q %s\n", res ? "OK" : "NULL");
    return(res);
}

int
strselect_p(struct connopt *options, PGresult *res, FILE *out) {
    int j, c, k, f;

    if  (v_flag > 3) fprintf( stderr, "+strselect_p out=%s\n"
                            , out ? "OK" : "NULL"
                            );
    if  (res == NULL) return(-1);
    j = 0;
    if  (options->tdl && options->tdl->tbeg && (f = strlen(options->tdl->tbeg))) {
        fwrite(options->tdl->tbeg, f, 1, out);
        j += f;
    }
    for (k = 0; k < PQntuples(res) && !feof(out) && !ferror(out); k++) {
        if  (options->tdl && options->tdl->rbeg && (f = strlen(options->tdl->rbeg))) {
            fwrite(options->tdl->rbeg, f, 1, out);
            j += f;
        }
        for (c = 0; c < PQnfields(res) && !feof(out) && !ferror(out); c++) {
            if  (options->tdl && options->tdl->cbeg && (f = strlen(options->tdl->cbeg))) {
                fwrite(options->tdl->cbeg, f, 1, out);
                j += f;
            }
            if  (!PQgetisnull(res, k, c)) {
                fwrite(PQgetvalue(res, k, c), (f = PQgetlength(res, k, c)), 1, out);
                j += f;
            }
            if  (options->tdl && options->tdl->cend && (f = strlen(options->tdl->cend))) {
                fwrite(options->tdl->cend, f, 1, out);
                j += f;
        }   }
        if  (options->tdl && options->tdl->rend && (f = strlen(options->tdl->rend))) {
            fwrite(options->tdl->rend, f, 1, out);
            j += f;
    }   }
    if  (options->tdl && options->tdl->tend && (f = strlen(options->tdl->tend))) {
        fwrite(options->tdl->tend, f, 1, out);
        j += f;
    }
    PQclear(res);
    if  (v_flag > 3)                                         fprintf( stderr, "-strselect_p %d\n", 0);
    return(0);
}
