/*-
 * Copyright (C) @BABOLO  2002 Dec 04
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: pgoblin.c,v 1.5 2002/12/23 04:40:24 babolo Exp $";
#endif /* not lint */
#include <sys/types.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgm.h"
#include "pgoblin.h"

int v_flag;
struct tabdelim tdl = {NULL, NULL, NULL, NULL, NULL, NULL};

void
usage(int ex) {
    fprintf(stderr, "pgoblin " VERS "  " DATE "
Usage:
pgoblin -[BDqsSv ][-c Table ][-C Table ][-d DataBase ][-h Hostname ][-p Port ][-U User ][query file]
    -d DataBase name
    -h Hostname of backend server
    -p Port
____-U Username
    -B bang strip
    -c COPY out
    -C COPY in
    -D perform query without result
    -s SELECT
____-S SELECT BINARY emulation
    -q quiet
    -v verbose
");
    exit(ex);
}

int
main(int argc, char *argv[]) {
    struct connopt *options = malloc(sizeof(struct connopt));
    struct pgoblin *pgm;
    FILE *out = stdout;
    FILE *in = stdin;
    int c;

    if  (options == NULL)                                  errx(EX_UNAVAILABLE, "Malloc failed #1\n");
    options->dbname = NULL;
    options->host = NULL;
    options->port = NULL;
    options->username = NULL;
    options->query = NULL;
    options->cmd = cmd_null;
    options->tdl = &tdl;
    options->intran = 0;
    options->bang = 0;
    v_flag = 0;  /*   */
    while ((c = getopt(argc, argv, "Bc:C:d:Dh:p:qsSU:v?")) != -1) {
        switch (c) {
        case 'B': options->bang++;
                  break;
        case 'c': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_copyout;
                  options->query = optarg;
                  break;
        case 'C': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_copyin;
                  options->query = optarg;
                  break;
        case 'd': options->dbname = optarg;
                  break;
        case 'D': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_perform;
                  break;
        case 'h': options->host = optarg;
                  break;
        case 'p': options->port = optarg;
                  break;
        case 'q': v_flag--;
                  break;
        case 's': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_strselect;
                  break;
        case 'S': if (options->cmd != cmd_null) usage(EX_USAGE);
                  options->cmd = cmd_binselect;
                  break;
        case 'U': options->username = optarg;
                  break;
        case 'v': v_flag++;
                  break;
        case '?': usage(EX_OK);
        default:  usage(EX_USAGE);
    }   }
    argc -= optind;
    argv += optind;
    if  (argc > 0 && argv && *argv) {
        in = fopen(*argv, "r");
        if  (in == NULL)                                           err(EX_NOINPUT, "file %s:", *argv);
    }

    /* Prepare the query */
    switch (options->cmd) {
    case cmd_null:
        if  ((pgm = parser(options, in, NULL)) == NULL) {
                                 if (v_flag >= 0) errx(EX_DATAERR, "Parser failed"); exit(EX_DATAERR);
        }
        if  (execute(options, pgm, stdin, out)) {
                                if (v_flag >= 0) errx(EX_DATAERR, "Execute failed"); exit(EX_DATAERR);
        }
        break;
    case cmd_copyin:
        if  (copyin(options, in, NULL, NULL)) {
                                if (v_flag >= 0) errx(EX_DATAERR, "COPY in failed"); exit(EX_DATAERR);
        }
        break;
    case cmd_copyout:
        if  (copyout(options, in, NULL, out)) {
                               if (v_flag >= 0) errx(EX_DATAERR, "COPY out failed"); exit(EX_DATAERR);
        }
        break;
    case cmd_binselect:
        if  (binselect(options, in, NULL, out)) {
                          if (v_flag >= 0) errx(EX_DATAERR, "SELECT BINARY failed"); exit(EX_DATAERR);
        }
        break;
    case cmd_strselect:
        if  (strselect_p(options, strselect_q(options, in, NULL), out)) {
                                 if (v_flag >= 0) errx(EX_DATAERR, "SELECT failed"); exit(EX_DATAERR);
        }
        break;
    case cmd_perform:
        if  (perform(options, in, NULL)) {
                                if (v_flag >= 0) errx(EX_DATAERR, "perform failed"); exit(EX_DATAERR);
        }
    case cmd_bang:
        break;
    default:
        usage(EX_USAGE);
    }
    PQfinish(options->conn);
    exit(EX_OK);
}
