#!/bin/sh -e

prog=`dirname ${0}`
cd ${prog}
prog="`pwd`/pgoblin ${1}"
TF=`mktemp -t pgoblin` || exit 1
${prog} -D << EOF || true
DROP TABLE pgoblin_test_table
EOF

echo 'CREATE TABLE pgoblin_test_table'
cat << EOF > ${TF}
#!${prog} -D
CREATE TABLE pgoblin_test_table
     ( t text
     , i int
     )
EOF
chmod a+x ${TF}
${TF}

echo 'COPY in    :test	0&1'
cat << EOF > ${TF}
#!${prog} -BC pgoblin_test_table
test	0
test	1
EOF
chmod a+x ${TF}
${TF}

echo 'COPY in with \ :test	2'
cat << EOF > ${TF}
#!${prog} -BC pgoblin_test_table
test	2
EOF
echo -n '\' >> ${TF}
chmod a+x ${TF}
${TF}

echo 'COPY in with \. :test	3'
cat << EOF > ${TF}
#!${prog} -BC pgoblin_test_table
test	3
EOF
echo -n '\.' >> ${TF}
chmod a+x ${TF}
${TF}

echo 'COPY in with \.\n :test	4'
cat << EOF > ${TF}
#!${prog} -BC pgoblin_test_table
test	4
\.
EOF
chmod a+x ${TF}
${TF}

echo 'COPY in    :test	5'
echo -n 'test	5' | ${prog} -C pgoblin_test_table

echo 'COPY in with \n :test	6'
${prog} -C pgoblin_test_table << EOF
test	6
EOF

echo 'COPY in with \ :test	7'
(echo 'test	7' ; echo -n '\') | ${prog} -C pgoblin_test_table

echo 'COPY in with \. :test	8'
(echo 'test	8' ; echo -n '\.') | ${prog} -C pgoblin_test_table

echo 'COPY in with \.\n :test	9'
${prog} -C pgoblin_test_table << EOF
test	9
\.
EOF

echo 'diff with SELECT'
3<< EOF
|test|0|
|test|1|
|test|2|
|test|3|
|test|4|
|test|5|
|test|6|
|test|7|
|test|8|
|test|9|
EOF
cat << EOF > ${TF}
#!${prog} -s
SELECT '|', t, '|', i, '|
' FROM pgoblin_test_table
EOF
${TF} | diff /dev/fd/3 /dev/fd/0

echo 'diff with SELECT'
4<< EOF
|test|0|
|test|1|
|test|2|
|test|3|
|test|4|
|test|5|
|test|6|
|test|7|
|test|8|
|test|9|
EOF
${prog} -s << EOF | diff /dev/fd/4 /dev/fd/0
SELECT '|', t, '|', i, '|
' FROM pgoblin_test_table
EOF

echo 'diff with COPY out'
5<< EOF
test	0
test	1
test	2
test	3
test	4
test	5
test	6
test	7
test	8
test	9
EOF
${prog} -c pgoblin_test_table | diff /dev/fd/5 /dev/fd/0

echo 'diff with SELECT BINARY'
6<< "EOF"
begin 644 test
M=&5S=`````!T97-T`0```'1E<W0"````=&5S=`,```!T97-T!````'1E<W0%
C````=&5S=`8```!T97-T!P```'1E<W0(````=&5S=`D`````
`
end
EOF
cat << EOF > ${TF}
#!${prog} -S
SELECT * FROM pgoblin_test_table
EOF
${TF} | uuencode test | diff /dev/fd/6 /dev/fd/0

echo 'diff with SELECT BINARY'
7<< "EOF"
begin 644 test
M=&5S=`````!T97-T`0```'1E<W0"````=&5S=`,```!T97-T!````'1E<W0%
C````=&5S=`8```!T97-T!P```'1E<W0(````=&5S=`D`````
`
end
EOF
${prog} -S << EOF | uuencode test | diff /dev/fd/7 /dev/fd/0
SELECT * FROM pgoblin_test_table
EOF

echo 'Program'
8<< EOF
#!${prog}
#copyin 
COPY pgoblin_test_table FROM stdin
#strselect 
SELECT t, ' ', i, i * i, '
' FROM pgoblin_test_table
#copyout 
COPY pgoblin_test_table TO stdout
#perform 
DROP TABLE pgoblin_test_table
EOF
9<< EOF
test 00
test 11
test 24
test 39
test 416
test 525
test 636
test 749
test 864
test 981
tset 10100
test	0
test	1
test	2
test	3
test	4
test	5
test	6
test	7
test	8
test	9
tset	10
EOF
${prog} /dev/fd/8 << EOF | diff /dev/fd/9 /dev/fd/0
tset	10
EOF

rm -f ${TF}
